      subroutine fun (indic,x,F,resx,izs,rzs,dzs)
      implicit integer *4 (i-n)
      implicit real *8 (a-h,o-z)
      integer *4 izs(*)
      real *4 rzs(*)
      dimension x(*),resx(*),dzs(*)
C--------------------------------------------------------------
C     This subroutine computes value (F) and subgradient
C     (resx) at point (x) of function to be minimized, using 
C     arrays izs(*), rzs(*) and dzs(*).  
C     Parametr indic must be 0 if x is feasible; 1 if x is not 
C     feasible; otherwise there are errors in fun.
C=====CONSTANTS================================================
      Zero=0.0D0
      Half=0.5D0
      One=1.0D0
      Ten=1.0D1
      Hund=1.0D2
      Thous=1.0D3
      dinfty=1.0D31
C=====LOCAL CONTROL PARAMETERS=================================
C-----no debugging (idebf=0) / with debugging (idebf=1)
      idebf=0
C-----# of channel for protocol file, must be already open
      io=15
C=====MEMORY DISTRIBUTION======================================
C--------------------------------------------------------------
C     izs(1) ==>   Ms=0, izs(2) ==>    Ns, izs(3) ==> Kbl,
C     izs(4) ==>  MbMax, izs(5) ==> NbMax, izs(6) ==>  it,
C     izs(7) <==    kki, izs(8) ==>   Nxx, izs(9) ==> Nyy,
C     izs(10) not used,
C-----
C     izs(11) ==> lNN=0, izs(12) ==>  lMs=0, izs(13) ==>  lNs, 
C     izs(14) ==>   Muu, izs(15) ==>    Nuu, izs(16) ==>  Luu,
C     izs(17) ==> ifbeg, izs(18) ==> jbegin, izs(19) ==> jend,
C     izs(20) ==> nfbeg
C-----
C     No MN- and Ms-block, only Ns-block: Ms = 0, lMN = lMs = 0.
C     Nyy*Ns <- size of Ns-block, Ns <- dimension of x,
C     Kbl <- # of diagonal blocks, it <- # of Level iterations,
C     MbMax,NbMax <- maximal sizes of diagonal blocks,
C     Nyy * Nxx <- sizes of the initial LP problem,
C     lNs <- # of non-zero elements in Ns-block,
C     Muu <- maximal size of rows in diagonal blocks (Muu=Mbmax),
C     Nuu <- maximal size of variables for Powell,
C     Luu <- maximal # of elements for Powell, Luu=Muu * Nuu.
C--------------------------------------------------------------
      Ms=izs(1)
      Ns=izs(2)
      Kbl=izs(3)
      Mbmax=izs(4)
      Nbmax=izs(5)
      it=izs(6)
      Nxx=izs(8)
      Nyy=izs(9)
      Nzz=Nxx+Nyy
      lMN=izs(11)
      lMs=izs(12)
      lNs=izs(13)
      Muu=izs(14)
      Nuu=izs(15)
      Luu=izs(16)
C--------------------------------------------------------------
C     ifbeg,jbegin,nfbeg - begin of diagonal blocks' integer,
C     real *4 and real *8 information, jend = Mdzs
C--------------------------------------------------------------
      ifbeg=izs(17)
      jbegin=izs(18)
      jend=izs(19)
      nfbeg=izs(20)
C-----the lengh of working integer *4 array for Powell
      iwork=Nuu
      irab=ifbeg-Nuu
C-----the lengh of working real *8 array for Powell
      jwork=3*Nuu*Nuu/2+10*Nuu+2*(Muu+1)+1
      jrab=jbegin-jwork
C-----storing the current point 
      jzz=jrab-Nxx 
      do j=1,Ns
         dzs(jzz+j)=x(j)
      enddo
C--------------------------------------------------------------
C     Storing BB = dzs(jbh), CC = dzs(jch) and their iterative
C     modifications
C--------------------------------------------------------------
      jcc=jzz-Nxx
      jbb=jcc-Nyy
      jch=jbb-Nxx
      jbh=jch-Nyy
C-----
      if (it.le.1.and.idebf.ne.0) then
         write (io,*)
         write (io,*) ' Ms,Ns,Kbl,MbMax,NbMax,Nxx,Nyy,Nzz=',
     *                  Ms,Ns,Kbl,MbMax,NbMax,Nxx,Nyy,Nzz
         write (io,*) ' irab,iwork,jrab,jwork=',
     *                  irab,iwork,jrab,jwork
         write (io,*) ' lNs,Muu,Nuu,Luu,jbegin,jend,it=',
     *                  lNs,Muu,Nuu,Luu,jbegin,jend,it
         write (io,*) ' jbh,jch,jbb,jcc,jzz=',
     *                  jbh,jch,jbb,jcc,jzz
      endif
C-----
      ifree=20
      jfree=10
      nfree=10
C--------------------------------------------------------------
C     Storing the non-zero elements of Ns-block
C--------------------------------------------------------------
      if (lNs.ne.0) then
         ikNs=ifree
         iaNs=ikNs+(Ns+1)
         ifree=iaNs+lNs
         jaNs=jfree
         jfree=jaNs+lNs
      endif
C--------------------------------------------------------------
C     Storing lBlk,Kineq,Mu,Nu,Lu,ibegi,ibegd,ibegr
C     for Kbl diagonal blocks:
C     lBlk <- # of non-zero elements in the block,
C     kineq = Mbl = Mu <- # of inequalities,
C     Nbl <- # of variables in the block, Nu=Nbl+Mu,
C     Lu = Mu * Nu <- for matrix for Solpwl (Powell),
C     ibegi,ibegd,ibegr <- begins of integer, real *4 and
C     real *8 blocks' information.
C--------------------------------------------------------------
      ilBlk=ifree
      ikineq=ilBlk+Kbl
      iMu=ikineq+Kbl
      iNu=iMu+Kbl
      iLu=iNu+Kbl
      ibegi=iLu+Kbl
      ibegd=ibegi+Kbl
      ibegr=ibegd+Kbl
      ifree=ibegr+Kbl
C-----
      if (it.le.1.and.idebf.ne.0) then
         write (io,*)
         write (io,*) ' ikNs,iaNs,jaNs,lNs=',
     *                  ikNs,iaNs,jaNs,lNs
         write (io,*) ' ilBlk,ikineq,iMu,iNu,iLu=',
     *                  ilBlk,ikineq,iMu,iNu,iLu
         write (io,*) ' ibegi,ibegd,ibegr=',
     *                  ibegi,ibegd,ibegr
      endif
C--------------------------------------------------------------
C     dzs(1) => epsd, dzs(2:10) not used, 
C     rzs(*) not used    
C--------------------------------------------------------------
      epsd=dzs(1)
      epsd2=epsd*epsd
C-----
      if (it.le.1.and.idebf.ne.0) then
         write (io,*)
         write (io,*) ' epsd=',epsd
      endif
C--------------------------------------------------------------
C     initial values of indic and 
C     kki <- # the sum of Oracle iterations 
C--------------------------------------------------------------
      indic=0
      kki=0
C-----F1 = <CC,x>
      F1=Zero
      if (Ns.ne.0) then
         do j=1,Ns
            w1=dzs(jch+j)
            w2=x(j)
            F1=F1+w1*w2
         enddo
      endif
C-----
      if (it.le.1.and.idebf.ne.0) then
         if (lNs.ne.0) then
            write (io,*)
            write (io,*) ' d(Ns)='
            write (io,*) (dzs(jaNs+l),l=1,lNs)
         endif
      endif
C--------------------------------------------------------------
C     To determine arrays Bbb = BB-A*x, 
C     Ccc = CC, where A <- Ns-block
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      call bcbar (Nxx,Nyy,x,dzs(jbh+1),dzs(jch+1),Ns,
     1            lNs,izs(ikNs+1),izs(iaNs+1),dzs(jaNs+1),
     2            dzs(jbb+1),dzs(jcc+1),idebf,io)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      if (idebf.ne.0) then
         write (io,*)
         write (io,*) ' Bbb='
         write (io,*) (dzs(jbb+i),i=1,Nyy)
         write (io,*) ' Ccc='
         write (io,*) (dzs(jcc+i),i=1,Nxx)
      endif
C--------------------------------------------------------------
C     Initial resx := -ccc, 
C     nodom <- # of inconsistant blocks,
C--------------------------------------------------------------
      F2=Zero
      F=-(F1+F2)
      F0=F
      if (Ns.ne.0) then
         do j=1,Ns
            resx(j)=-dzs(jcc+j)
         enddo
      endif
      nodom=0
C-----
      if (idebf.ne.0) then
         write (io,*) ' F1,F2,F=',F1,F2,F
         if (Ns.ne.0) then
            write (io,*) ' Resx='
            write (io,*) (resx(j),j=1,Ns)
         endif
      endif
C=====For Diagonal Blocks======================================
      Mblk0=Ms
      Nblk0=Ns
      do k=1,Kbl
C--------problem k
         if (idebf.ne.0) then
            write (io,*) '------------------------'
            write (io,*) ' *** Block k=',k,' it=',it
         endif
C--------------------------------------------------------------
C        Determine (unpack) main parameters of the k-th block:
C        lBlk; KinBlk = Mu = Mbl; Nu=Nbl + Mu; Lu = Mu * Nu;
C        ikBlk(1:Nbl+1) - begins of the block's columns;
C        jaBlk(1:lBlk) - non-zero elements of the block;
C        iaBlk(1:lBlk) - numbers of rows in initial LP problem.
C--------------------------------------------------------------
         lBlk=izs(ilBlk+k)
         KinBlk=izs(ikineq+k)
         KeqBlk=0
         Mu=izs(iMu+k)
         Mbl=Mu
         Nu=izs(iNu+k)
         NBl=Nu-Mbl
         Nu2=Nu*Nu
         Lu=izs(iLu+k)
         if (idebf.ne.0) then
            write (io,*)
            write (io,*) ' lBlk,KinBlk,Mu,Nu,Lu,Mbl,Nbl=',
     *                     lBlk,KinBlk,Mu,Nu,Lu,Mbl,Nbl
         endif
         ibeg=izs(ibegi+k)
         ikBlk=ibeg
         iaBlk=ikBlk+(Nbl+1)
         ifree=iaBlk+lBlk
         if (it.le.1.and.idebf.ne.0) then
            write (io,*)
            write (io,*) ' ikBlk,iaBlk=',ikBlk,iaBlk
         endif
C--------------------------------------------------------------
C        To store some arrays for Powell:
C        Nu = size of vectors xP,yP,jlft,jrht,dP;
C        cP <- Nu*Nu-diagonal matrix, aP <- Mu*Nu-matrix,
C        Mu = size of bP, the size of zP = Mu + 2 * Nu.
C--------------------------------------------------------------
         jbeg=izs(ibegd+k)
         jaBlk=jbeg
         jfree=jaBlk+lBlk
         jxP=jfree
         jyP=jxP+Nu
         jlft=jyP+Mu
         jrht=jlft+Nu
         jcP=jrht+Nu
         jdP=jcP+Nu*Nu
         jbP=jdP+Nu
         jaP=jbP+Mu
         jzP=jaP+Mu*Nu
         jfree=jzP+Mu+Nu+Nu
C--------
         if (it.le.1.and.idebf.ne.0) then
            write (io,*)
            write (io,*) ' jaBlk,jxP,jyP,jlft,jrht=',
     *                     jaBlk,jxP,jyP,jlft,jrht
            write (io,*) ' jcP,jdP,jbP,jaP,jzP=',
     *                     jcP,jdP,jbP,jaP,jzP
         endif
C--------
         nbeg=izs(ibegr+k)
C--------------------------------------------------------------
C        Correct the information for Powell,
C        set bbb(i) (ccc(j) is not changed)
C--------------------------------------------------------------
         do j=1,Nbl
            jj=jdP+j
            dzs(jj)=Zero
         enddo
         do i=1,Mu
            jj=jdP+Nbl+i
            dzs(jj)=One
            jj=jrht+Nbl+i
            dzs(jj)=dinfty
         enddo
         do i=1,Mu
            ii=jbP+i
            dzs(ii)=+dzs(jbb+Mblk0+i)
         enddo
C--------
         if (idebf.ne.0) then
            write (io,*)
            write (io,*) ' d='
            write (io,*) (dzs(jdP+i),i=1,Nu)
            write (io,*) ' b='
            write (io,*) (dzs(jbP+i),i=1,Mu)
         endif
C=======Computing minimum of the model LP(x)===================
         if (idebf.ne.0) then
            write (io,*)
            write (io,*) k,' +++ Before Solpwl +++'
         endif
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
         call solpwl (k,nodom,Nbl,Mu,Nu,KeqBlk,iter,ifail,
     1                idebf,io,dzs(jcP+1),dzs(jdP+1),
     2                dzs(jcc+Nblk0+1),dzs(jaP+1),
     3                dzs(jbP+1),dzs(jlft+1),dzs(jrht+1),
     4                dzs(jxP+1),dzs(jzP+1),dzs(jrab+1),
     5                jwork,izs(irab+1),iwork)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
         Fk=Zero
         do i=1,Nu
            w1=dzs(jxP+i)
            w2=dzs(jdP+i)
            w=w1*w2
            Fk=Fk+w
         enddo
C--------
         if (idebf.ne.0) then
            write (io,*)
            write (io,*) k,' +++ After Solpwl +++'
            write (io,*) k,' iter,ifail,Fk,it=',
     *                       iter,ifail,Fk,it
         endif
         if (ifail.gt.1.or.ifail.lt.0) then
            indic=2
            ifail=indic
            goto 501
         endif
         do i=1,Mu
            dzs(jyP+i)=+dzs(jzP+i)
         enddo
C--------------------------------------------------------------
  200    continue
         if (idebf.ne.0) then
            write (io,*)
            write (io,*) ' X answ='
            write (io,*) (dzs(jxP+i),i=1,Nu)
            write (io,*)
            write (io,*) ' Y answ='
            write (io,*) (dzs(jyP+i),i=1,Mu)
         endif
C--------------------------------------------------------------
C        If for the 1-st time block # k is inconsistent,
C        then using Phi(x) instead of f(x)
C--------------------------------------------------------------
         if (ifail.eq.1.and.nodom.eq.0) then
            if (Ns.ne.0) then
               do j=1,Ns
                 resx(j)=Zero
               enddo
            endif
            F=Zero
            indic=1
         endif
C--------Storing the solution of the k-th block
         do j=1,Nu
            dzs(jzz+Nblk0+j)=dzs(jxP+j)
         enddo
C--------
         if (ifail.eq.1) nodom=nodom+1
         kki=kki+iter
C--------
         if (idebf.ne.0) then
            write (io,*)
            write (io,*) ' k,it=',k,it
            write (io,*) ' xblock='
            write (io,*) (dzs(jxP+j),j=1,Nu)
            write (io,*)
            write (io,*) ' y='
            write (io,*) (dzs(jyP+i),i=1,Mu)
            write (io,*)
            write (io,*) ' F,Fk,ibx=',F,Fk
            write (io,*) ' ynew='
            write (io,*) (dzs(jyP+i),i=1,Mbl)
         endif
C========Subgradient resx======================================
         if (nodom.eq.0.or.ifail.ne.0) then
C--------correct the subgradient------------------------------- 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
            call rescon (Ns,Mbl,Mblk0,resx,dzs(jyP+1),lNs,
     1                   izs(ikNs+1),izs(iaNs+1),dzs(jaNs+1),
     2                   idebf,io)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
            F=F+Fk
            if (idebf.ne.0) write (io,*) ' F=',F
         endif
C--------    
         if (idebf.ne.0) then
            if (Ns.ne.0) then
               write (io,*)
               write (io,*) ' resx= '
               write (io,*) (resx(j),j=1,Ns)
            endif
         endif
         Mblk0=Mblk0+Mbl
         Nblk0=Nblk0+NBl
      enddo                   !  k
C=====ANSWER===================================================
  501 izs(7)=kki
C-----
      if (idebf.ne.0) then
         write (io,*) ' F0,F1,F2,F=',F0,F1,F2,F
         if (Ns.ne.0) then
            write (io,*)
            write (io,*) ' x= '
            write (io,*) (dzs(j),j=1,Nxx)
            write (io,*) ' resx= '
            write (io,*) (resx(j),j=1,Ns)
         endif
      endif
      return
      end
C--------------------------------------------------------------
      subroutine rescon (Ns,Mbl,Mblk0,resx,y,lNs,icNs,
     *                   iaNs,daNs,ideb,io)
      implicit integer *4 (i-n)
      implicit real *8 (a-h,o-z)
      dimension resx(*),y(*),icNs(*),iaNs(*),daNs(*)
C--------------------------------------------------------------
C     This subroutine determines recursively
C     resx := resx + y * A, where vector
C     y <- the dual solution of the k-th block from Solpwl. 
C--------------------------------------------------------------
      Zero=0.0D0
      if (lNs.ne.0) then
         i1=Mblk0
         i2=i1+Mbl
         do j=1,Ns
            lj1=icNs(j)+1
            lj2=icNs(j+1)
            if (lj1.le.lj2) then
               do l=lj1,lj2
                  i=iaNs(l)
                  if (i.ge.i1+1.and.i.le.i2) then
                     w=daNs(l)
                     yy=y(i-i1)
                     if (yy.ne.Zero) resx(j)=resx(j)+w*yy
                  endif
               enddo
            endif
         enddo
      endif
C-----
      if (ideb.ne.0) then
C        write (io,*) ...
      endif
      return
      end
C--------------------------------------------------------------
      subroutine solpwl (k,nodom,Nbl,Mu,Nu,KeqBlk,iter,ifail,
     1                   ideb,io,c,d,cena,a,b,xl,xu,x,z,
     2                   wR,jwork,iR,iwork)
      implicit integer *4 (i-n)
      implicit real *8 (a-h,o-z)
      dimension c(*),d(*),cena(*),b(*),a(*),xl(*),xu(*)
      dimension x(*),z(*),wR(*),iR(*)
      common /cmache/ epsp /powit/ itrpw 
      character *8 simpwl
C--------------------------------------------------------------
C     This subroutine determines:
C     A) The k-th Oracle problem is consistent. Then
C        if nodom = 0, then additionally the solution of 
C        the current Oracle problem is found. 
C     B) The problem is not consistent. Then nodom:=nodom+1.
C--------------------------------------------------------------
      Zero=0.0D0
      One=1.0D0
      dinfty=1.0D31
      iprint=0
      Epsadd=1.0D-8
C-----Determine the consistence ot the Oracle problem
      simpwl='Powell-1'
      do j=1,Nbl
         d(j)=Zero
      enddo
      do i=1,Mu
         d(Nbl+i)=One
         xu(Nbl+i)=dinfty
      enddo
      me=KeqBlk
      mmax=Mu
      nmax=Nu
      mnn=Mu+Nu+Nu
      iR(1)=0
C-----
      if (ideb.ne.0) then
         write (io,*)
         write (io,*) k,' --- Before Powell-1 ---'
         write (io,*) k,' c='
         write (io,*) k,(c(j),j=1,Nu*Nu)
         write (io,*) k,' d='
         write (io,*) k,(d(j),j=1,Nu)
         write (io,*) k,' b='
         write (io,*) k,(b(i),i=1,Mu)
         write (io,*) k,' a='
         write (io,*) k,(a(i),i=1,Mu*Nu)
         write (io,*) k,' xl='
         write (io,*) k,(xl(j),j=1,Nu)
         write (io,*) k,' xu='
         write (io,*) k,(xu(j),j=1,Nu)
         write (io,*) k,' iR(1)=',iR(1)
      endif
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      call ql0001 (Mu,me,mmax,Nu,nmax,mnn,c,d,a,b,xl,xu,
     *             x,z,io,ifail,iprint,wr,jwork,ir,iwork)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      if (ideb.ne.0) then
         write (io,*)
         write (io,*) k,' --- After Powell-1 ---'
         write (io,*) k,' ifail,itrpw=',ifail,itrpw
         write (io,*) k,' x='
         write (io,*) k,(x(j),j=1,Nu)
         write (io,*) k,' z='
         write (io,*) k,(z(i),i=1,mnn)
      endif
C--------------------------------------------------------------

C     ifail = 0 <== the problem is consistent,
C     ifail = 1 <== the problem is not consistent,
C                otherwise bad.
C--------------------------------------------------------------
      if (ifail.ne.0) then
         ifail=-1
         goto 200
      endif
      iter=iabs(itrpw)
      Vsum=Zero
      do i=1,Mu
         Vsum=Vsum+x(Nbl+i)
      enddo
      if (Vsum.gt.Epsadd) then
         ifail=1
         goto 200
      endif
C=====The problem is consistent================================
  100 continue
C-----There is no use to solve the Oracle problem   
      if (nodom.ne.0) goto 200
C-----Solve the k-th Oracle problem
      do j=1,Nbl
         d(j)=-cena(j)
      enddo
      do i=1,Mu
         d(Nbl+i)=Zero
         xu(Nbl+i)=Zero
      enddo
      simpwl='Powell-2'
      me=KeqBlk
      mmax=Mu
      nmax=Nu
      mnn=Mu+Nu+Nu
      iR(1)=0
C-----
      if (ideb.ne.0) then
         write (io,*)
         write (io,*) k,' --- Before ',simpwl,' ---'
         write (io,*) k,' c='
         write (io,*) k,(c(j),j=1,Nu*Nu)
         write (io,*) k,' d='
         write (io,*) k,(d(j),j=1,Nu)
         write (io,*) k,' b='
         write (io,*) (k,b(i),i=1,Mu)
         write (io,*) k,' a='
         write (io,*) k,(a(i),i=1,Nu*Mu)
         write (io,*) k,' xl='
         write (io,*) k,(xl(j),j=1,Nu)
         write (io,*) k,' xu='
         write (io,*) k,(xu(j),j=1,Nu)
         write (io,*) k,' iR(1)=',iR(1)
      endif
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      call ql0001 (Mu,me,mmax,Nu,nmax,mnn,c,d,a,b,xl,xu,
     *             x,z,io,ifail,iprint,wR,jwork,iR,iwork)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      if (ideb.ne.0) then
         write (io,*)
         write (io,*) k,' --- After ',simpwl,' ---'
         write (io,*) k,' ifail,itrpw=',ifail,itrpw
         write (io,*) k,' x='
         write (io,*) k,(x(j),j=1,Nu)
         write (io,*) k,' z='
         write (io,*) k,(z(i),i=1,mnn)
      endif
C-----
      if (ifail.ne.0) then
         ifail=-2
         goto 200
      endif
      iter=iter+iabs(itrpw)
C-----
  200 return
      end
C--------------------------------------------------------------
      subroutine bcbar (Nx,Ny,x,BB,CC,Ns,lNs,icNs,
     *                  iaNs,daNs,bbb,ccc,ideb,io)
      implicit integer *4 (i-n)
      implicit real *8 (a-h,o-z)
      dimension BB(*),CC(*),x(*),icNs(*),iaNs(*),daNs(*),
     *          bbb(*),ccc(*)
C--------------------------------------------------------------
C     This subroutine calculates vectors
C     bbb(1:Ny) := BB - Ax, ccc(1:Nx) := CC, at point x, 
C     where A is Ns-block, given by the parameters Ns, lNs 
C     and by the arrays icNs, iaNs, daNs.
C--------------------------------------------------------------
      Zero=0.0D0
      do i=1,Ny
         bbb(i)=BB(i)
      enddo
      do j=1,Nx
         ccc(j)=CC(j)
      enddo
      if (lNs.ne.0) then
         do j=1,Ns
            xx=x(j)
            if (xx.ne.Zero) then
               lj1=icNs(j)+1
               lj2=icNs(j+1)
               if (lj1.le.lj2) then
                  do l=lj1,lj2
                     i=iaNs(l)
                     ww=daNs(l)
                     if (ww.ne.Zero) bbb(i)=bbb(i)-ww*xx
                  enddo
               endif
            endif
         enddo
      endif
C-----
      if (ideb.gt.1) then
         write (io,*)
         write (io,*) ' bbb='
         write (io,*) (bbb(i),i=1,Ny)
         write (io,*) ' ccc='
         write (io,*) (ccc(i),i=1,Nx)
      endif
      return
      end
